@@ -7,7 +7,7 @@
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,12 +20,61 @@
 # Author: David Zeuthen <davidz@redhat.com>
 
 import distutils.version
+import os
+import sys
+
+
+# pylint: disable=too-few-public-methods
+class Color:
+    """ANSI Terminal colors"""
+
+    GREEN = "\033[1;32m"
+    BLUE = "\033[1;34m"
+    YELLOW = "\033[1;33m"
+    RED = "\033[1;31m"
+    END = "\033[0m"
+
+
+def print_color(msg, color=Color.END, prefix="MESSAGE"):
+    """Print a string with a color prefix"""
+    if os.isatty(sys.stderr.fileno()):
+        real_prefix = "{start}{prefix}{end}".format(
+            start=color, prefix=prefix, end=Color.END
+        )
+    else:
+        real_prefix = prefix
+    sys.stderr.write("{prefix}: {msg}\n".format(prefix=real_prefix, msg=msg))
+
+
+def print_error(msg):
+    """Print an error, and terminate"""
+    print_color(msg, color=Color.RED, prefix="ERROR")
+    sys.exit(1)
+
+
+def print_warning(msg, fatal=False):
+    """Print a warning, and optionally terminate"""
+    if fatal:
+        color = Color.RED
+        prefix = "ERROR"
+    else:
+        color = Color.YELLOW
+        prefix = "WARNING"
+    print_color(msg, color, prefix)
+    if fatal:
+        sys.exit(1)
+
+
+def print_info(msg):
+    """Print a message"""
+    print_color(msg, color=Color.GREEN, prefix="INFO")
+
 
 def strip_dots(s):
-    ret = ''
+    ret = ""
     force_upper = False
     for c in s:
-        if c == '.':
+        if c == ".":
             force_upper = True
         else:
             if force_upper:
@@ -35,19 +84,21 @@ def strip_dots(s):
                 ret += c
     return ret
 
+
 def dots_to_hyphens(s):
-    return s.replace('.', '-')
+    return s.replace(".", "-")
+
 
 def camel_case_to_uscore(s):
-    ret = ''
+    ret = ""
     insert_uscore = False
     prev_was_lower = False
-    initial = True;
+    initial = True
     for c in s:
         # Keep initial underscores in camel case
-        if initial and c == '_':
-            ret += '_'
-            continue;
+        if initial and c == "_":
+            ret += "_"
+            continue
         initial = False
 
         if c.isupper():
@@ -57,16 +108,22 @@ def camel_case_to_uscore(s):
         else:
             prev_was_lower = True
         if insert_uscore:
-            ret += '_'
+            ret += "_"
         ret += c.lower()
         insert_uscore = False
     return ret
 
+
+def uscore_to_camel_case(s):
+    return "".join([s[0].upper() + s[1:].lower() if s else "_" for s in s.split("_")])
+
+
 def is_ugly_case(s):
-    if s and s.find('_') > 0:
+    if s and s.find("_") > 0:
         return True
     return False
 
+
 def lookup_annotation(annotations, key):
     if annotations:
         for a in annotations:
@@ -74,29 +131,39 @@ def lookup_annotation(annotations, key):
                 return a.value
     return None
 
+
 def lookup_docs(annotations):
-    s = lookup_annotation(annotations, 'org.gtk.GDBus.DocString')
-    if s == None:
-        return ''
+    s = lookup_annotation(annotations, "org.gtk.GDBus.DocString")
+    if s is None:
+        return ""
     else:
         return s
 
+
 def lookup_since(annotations):
-    s = lookup_annotation(annotations, 'org.gtk.GDBus.Since')
-    if s == None:
-        return ''
+    s = lookup_annotation(annotations, "org.gtk.GDBus.Since")
+    if s is None:
+        return ""
     else:
         return s
 
+
 def lookup_brief_docs(annotations):
-    s = lookup_annotation(annotations, 'org.gtk.GDBus.DocString.Short')
-    if s == None:
-        return ''
+    s = lookup_annotation(annotations, "org.gtk.GDBus.DocString.Short")
+    if s is None:
+        return ""
     else:
         return s
 
+
 def version_cmp_key(key):
-    # If the 'since' version is empty put a 0 in its place as this will
+    # If the 'since' version is 'UNRELEASED', compare higher than anything else
+    # If it is empty put a 0 in its place as this will
     # allow LooseVersion to work and will always compare lower.
-    v = key[0] if key[0] else '0'
+    if key[0] == "UNRELEASED":
+        v = "9999"
+    elif key[0]:
+        v = str(key[0])
+    else:
+        v = "0"
     return (distutils.version.LooseVersion(v), key[1])
