@@ -2,6 +2,8 @@
  * Copyright 2015 Lars Uebernickel
  * Copyright 2015 Ryan Lortie
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -62,6 +64,7 @@ enum
 {
   PROP_0,
   PROP_ITEM_TYPE,
+  PROP_N_ITEMS,
   N_PROPERTIES
 };
 
@@ -70,6 +73,8 @@ static void g_list_store_iface_init (GListModelInterface *iface);
 G_DEFINE_TYPE_WITH_CODE (GListStore, g_list_store, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, g_list_store_iface_init));
 
+static GParamSpec *properties[N_PROPERTIES] = { NULL, };
+
 static void
 g_list_store_items_changed (GListStore *store,
                             guint       position,
@@ -85,6 +90,8 @@ g_list_store_items_changed (GListStore *store,
     }
 
   g_list_model_items_changed (G_LIST_MODEL (store), position, removed, added);
+  if (removed != added)
+    g_object_notify_by_pspec (G_OBJECT (store), properties[PROP_N_ITEMS]);
 }
 
 static void
@@ -111,6 +118,10 @@ g_list_store_get_property (GObject    *object,
       g_value_set_gtype (value, store->item_type);
       break;
 
+    case PROP_N_ITEMS:
+      g_value_set_uint (value, g_sequence_get_length (store->items));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -153,9 +164,22 @@ g_list_store_class_init (GListStoreClass *klass)
    *
    * Since: 2.44
    **/
-  g_object_class_install_property (object_class, PROP_ITEM_TYPE,
+  properties[PROP_ITEM_TYPE] =
     g_param_spec_gtype ("item-type", "", "", G_TYPE_OBJECT,
-                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GListStore:n-items:
+   *
+   * The number of items contained in this list store.
+   *
+   * Since: 2.74
+   **/
+  properties[PROP_N_ITEMS] =
+    g_param_spec_uint ("n-items", "", "", 0, G_MAXUINT, 0,
+                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
 }
 
 static GType
@@ -281,8 +305,8 @@ g_list_store_insert (GListStore *store,
  * g_list_store_insert_sorted:
  * @store: a #GListStore
  * @item: (type GObject): the new item
- * @compare_func: (scope call): pairwise comparison function for sorting
- * @user_data: (closure): user data for @compare_func
+ * @compare_func: (scope call) (closure user_data): pairwise comparison function for sorting
+ * @user_data: user data for @compare_func
  *
  * Inserts @item into @store at a position to be determined by the
  * @compare_func.
@@ -321,8 +345,8 @@ g_list_store_insert_sorted (GListStore       *store,
 /**
  * g_list_store_sort:
  * @store: a #GListStore
- * @compare_func: (scope call): pairwise comparison function for sorting
- * @user_data: (closure): user data for @compare_func
+ * @compare_func: (scope call) (closure user_data): pairwise comparison function for sorting
+ * @user_data: user data for @compare_func
  *
  * Sort the items in @store according to @compare_func.
  *
@@ -495,18 +519,30 @@ g_list_store_splice (GListStore *store,
   g_list_store_items_changed (store, position, n_removals, n_additions);
 }
 
+static gboolean
+simple_equal (gconstpointer a,
+              gconstpointer b,
+              gpointer equal_func)
+{
+  return ((GEqualFunc) equal_func) (a, b);
+}
+
 /**
  * g_list_store_find_with_equal_func:
  * @store: a #GListStore
- * @item: (type GObject): an item
+ * @item: (type GObject) (nullable): an item
  * @equal_func: (scope call): A custom equality check function
  * @position: (out) (optional): the first position of @item, if it was found.
  *
  * Looks up the given @item in the list store by looping over the items and
- * comparing them with @compare_func until the first occurrence of @item which
+ * comparing them with @equal_func until the first occurrence of @item which
  * matches. If @item was not found, then @position will not be set, and this
  * method will return %FALSE.
  *
+ * @item is always passed as second parameter to @equal_func.
+ *
+ * Since GLib 2.76 it is possible to pass `NULL` for @item.
+ *
  * Returns: Whether @store contains @item. If it was found, @position will be
  * set to the position where @item occurred for the first time.
  *
@@ -517,11 +553,44 @@ g_list_store_find_with_equal_func (GListStore *store,
                                    gpointer    item,
                                    GEqualFunc  equal_func,
                                    guint      *position)
+{
+  g_return_val_if_fail (equal_func != NULL, FALSE);
+
+  return g_list_store_find_with_equal_func_full (store, item, simple_equal,
+                                                 equal_func, position);
+}
+
+/**
+ * g_list_store_find_with_equal_func_full:
+ * @store: a #GListStore
+ * @item: (type GObject) (nullable): an item
+ * @equal_func: (scope call) (closure user_data): A custom equality check function
+ * @user_data: user data for @equal_func
+ * @position: (out) (optional): the first position of @item, if it was found.
+ *
+ * Like g_list_store_find_with_equal_func() but with an additional @user_data
+ * that is passed to @equal_func.
+ *
+ * @item is always passed as second parameter to @equal_func.
+ *
+ * Since GLib 2.76 it is possible to pass `NULL` for @item.
+ *
+ * Returns: Whether @store contains @item. If it was found, @position will be
+ * set to the position where @item occurred for the first time.
+ *
+ * Since: 2.74
+ */
+gboolean
+g_list_store_find_with_equal_func_full (GListStore     *store,
+                                        gpointer        item,
+                                        GEqualFuncFull  equal_func,
+                                        gpointer        user_data,
+                                        guint          *position)
 {
   GSequenceIter *iter, *begin, *end;
 
   g_return_val_if_fail (G_IS_LIST_STORE (store), FALSE);
-  g_return_val_if_fail (g_type_is_a (G_OBJECT_TYPE (item), store->item_type),
+  g_return_val_if_fail (item == NULL || g_type_is_a (G_OBJECT_TYPE (item), store->item_type),
                         FALSE);
   g_return_val_if_fail (equal_func != NULL, FALSE);
 
@@ -536,7 +605,7 @@ g_list_store_find_with_equal_func (GListStore *store,
       gpointer iter_item;
 
       iter_item = g_sequence_get (iter);
-      if (equal_func (iter_item, item))
+      if (equal_func (iter_item, item, user_data))
         {
           if (position)
             *position = g_sequence_iter_get_position (iter);
