@@ -2,6 +2,8 @@
  *
  * Copyright (C) 2010 Collabora, Ltd.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -25,7 +27,9 @@
 
 #include "gasyncresult.h"
 #include "ginetaddress.h"
+#include "gioerror.h"
 #include "glibintl.h"
+#include "glib-private.h"
 #include "gnetworkaddress.h"
 #include "gnetworkingprivate.h"
 #include "gproxy.h"
@@ -87,45 +91,34 @@ struct _GProxyAddressEnumeratorPrivate
   gboolean                  supports_hostname;
   GList                    *next_dest_ip;
   GError                   *last_error;
+
+  /* ever_enumerated is TRUE after we've returned a result for the first time
+   * via g_proxy_address_enumerator_next() or _next_async(). If FALSE, we have
+   * never returned yet, and should return an error if returning NULL because
+   * it does not make sense for a proxy resolver to return NULL except on error.
+   * (Whereas a DNS resolver would return NULL with no error to indicate "no
+   * results", a proxy resolver would want to return "direct://" instead, so
+   * NULL without error does not make sense for us.)
+   *
+   * But if ever_enumerated is TRUE, then we must not report any further errors
+   * (except for G_IO_ERROR_CANCELLED), because this is an API contract of
+   * GSocketAddressEnumerator.
+   */
+  gboolean                  ever_enumerated;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GProxyAddressEnumerator, g_proxy_address_enumerator, G_TYPE_SOCKET_ADDRESS_ENUMERATOR)
 
 static void
 save_userinfo (GProxyAddressEnumeratorPrivate *priv,
-	       const gchar *proxy)
+               const gchar *proxy)
 {
-  gchar *userinfo;
+  g_clear_pointer (&priv->proxy_username, g_free);
+  g_clear_pointer (&priv->proxy_password, g_free);
 
-  if (priv->proxy_username)
-    {
-      g_free (priv->proxy_username);
-      priv->proxy_username = NULL;
-    }
-
-  if (priv->proxy_password)
-    {
-      g_free (priv->proxy_password);
-      priv->proxy_password = NULL;
-    }
-  
-  if (_g_uri_parse_authority (proxy, NULL, NULL, &userinfo, NULL))
-    {
-      if (userinfo)
-	{
-	  gchar **split = g_strsplit (userinfo, ":", 2);
-
-	  if (split[0] != NULL)
-	    {
-	      priv->proxy_username = g_uri_unescape_string (split[0], NULL);
-	      if (split[1] != NULL)
-		priv->proxy_password = g_uri_unescape_string (split[1], NULL);
-	    }
-
-	  g_strfreev (split);
-	  g_free (userinfo);
-	}
-    }
+  g_uri_split_with_user (proxy, G_URI_FLAGS_HAS_PASSWORD, NULL,
+                         &priv->proxy_username, &priv->proxy_password,
+                         NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 }
 
 static void
@@ -166,9 +159,13 @@ next_enumerator (GProxyAddressEnumeratorPrivate *priv)
       else
 	{
 	  GError *error = NULL;
+	  int default_port;
 
-	  connectable = g_network_address_parse_uri (priv->proxy_uri, 0, &error);
+	  default_port = GLIB_PRIVATE_CALL (g_uri_get_default_scheme_port) (priv->proxy_type);
+	  if (default_port == -1)
+	    default_port = 0;
 
+	  connectable = g_network_address_parse_uri (priv->proxy_uri, default_port, &error);
 	  if (error)
 	    {
 	      g_warning ("Invalid proxy URI '%s': %s",
@@ -196,8 +193,9 @@ g_proxy_address_enumerator_next (GSocketAddressEnumerator  *enumerator,
   GSocketAddress *result = NULL;
   GError *first_error = NULL;
 
-  if (priv->proxies == NULL)
+  if (!priv->ever_enumerated)
     {
+      g_assert (priv->proxies == NULL);
       priv->proxies = g_proxy_resolver_lookup (priv->proxy_resolver,
 					       priv->dest_uri,
 					       cancellable,
@@ -205,7 +203,10 @@ g_proxy_address_enumerator_next (GSocketAddressEnumerator  *enumerator,
       priv->next_proxy = priv->proxies;
 
       if (priv->proxies == NULL)
-	return NULL;
+	{
+	  priv->ever_enumerated = TRUE;
+	  return NULL;
+	}
     }
 
   while (result == NULL && (*priv->next_proxy || priv->addr_enum))
@@ -287,8 +288,12 @@ g_proxy_address_enumerator_next (GSocketAddressEnumerator  *enumerator,
 	}
       dest_protocol = g_uri_parse_scheme (priv->dest_uri);
 		 		  
-      g_return_val_if_fail (G_IS_INET_SOCKET_ADDRESS (priv->proxy_address),
-			    NULL);
+      if (!G_IS_INET_SOCKET_ADDRESS (priv->proxy_address))
+        {
+	  g_free (dest_hostname);
+	  g_free (dest_protocol);
+        }
+      g_return_val_if_fail (G_IS_INET_SOCKET_ADDRESS (priv->proxy_address), NULL);
 
       inetsaddr = G_INET_SOCKET_ADDRESS (priv->proxy_address);
       inetaddr = g_inet_socket_address_get_address (inetsaddr);
@@ -315,29 +320,37 @@ g_proxy_address_enumerator_next (GSocketAddressEnumerator  *enumerator,
 	}
     }
 
-  if (result == NULL && first_error)
+  if (result == NULL && first_error && (!priv->ever_enumerated || g_error_matches (first_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)))
     g_propagate_error (error, first_error);
   else if (first_error)
     g_error_free (first_error);
 
+  if (result == NULL && error != NULL && *error == NULL && !priv->ever_enumerated)
+    g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, _("Unspecified proxy lookup failure"));
+
+  priv->ever_enumerated = TRUE;
+
   return result;
 }
 
-
-
 static void
 complete_async (GTask *task)
 {
   GProxyAddressEnumeratorPrivate *priv = g_task_get_task_data (task);
 
-  if (priv->last_error)
+  if (priv->last_error && (!priv->ever_enumerated || g_error_matches (priv->last_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)))
     {
       g_task_return_error (task, priv->last_error);
       priv->last_error = NULL;
     }
+  else if (!priv->ever_enumerated)
+    g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_FAILED, _("Unspecified proxy lookup failure"));
   else
     g_task_return_pointer (task, NULL, NULL);
 
+  priv->ever_enumerated = TRUE;
+
+  g_clear_error (&priv->last_error);
   g_object_unref (task);
 }
 
@@ -346,6 +359,7 @@ return_result (GTask *task)
 {
   GProxyAddressEnumeratorPrivate *priv = g_task_get_task_data (task);
   GSocketAddress *result;
+  gboolean is_inet_socket_address;
 
   if (strcmp ("direct", priv->proxy_type) == 0)
     {
@@ -377,7 +391,13 @@ return_result (GTask *task)
 	}
       dest_protocol = g_uri_parse_scheme (priv->dest_uri);
 
-      g_return_if_fail (G_IS_INET_SOCKET_ADDRESS (priv->proxy_address));
+      is_inet_socket_address = G_IS_INET_SOCKET_ADDRESS (priv->proxy_address);
+      if (!is_inet_socket_address)
+        {
+	  g_free (dest_hostname);
+	  g_free (dest_protocol);
+        }
+      g_return_if_fail (is_inet_socket_address);
 
       inetsaddr = G_INET_SOCKET_ADDRESS (priv->proxy_address);
       inetaddr = g_inet_socket_address_get_address (inetsaddr);
@@ -404,6 +424,7 @@ return_result (GTask *task)
 	}
     }
 
+  priv->ever_enumerated = TRUE;
   g_task_return_pointer (task, result, g_object_unref);
   g_object_unref (task);
 }
