@@ -1,6 +1,8 @@
 /* Testcase for bug in GIO function g_file_query_filesystem_info()
  * Author: Nelson Benítez León
  *
+ * SPDX-License-Identifier: LicenseRef-old-glib-tests
+ *
  * This work is provided "as is"; redistribution and modification
  * in whole or in part, in any medium, physical or electronic is
  * permitted without restriction.
@@ -135,6 +137,13 @@ test_filesystem_readonly (gconstpointer with_mount_monitor)
   if (! g_file_set_contents (file_in_mount, "Example", -1, NULL))
     {
       g_test_skip ("Failed to create file needed to proceed further with the test");
+
+      g_free (dir_mountpoint);
+      g_free (file_in_mount);
+      g_free (dir_to_mount);
+      g_free (curdir);
+      g_free (fusermount);
+      g_free (bindfs);
       return;
     }
 
@@ -147,8 +156,18 @@ test_filesystem_readonly (gconstpointer with_mount_monitor)
     {
       gchar *skip_message = g_strdup_printf ("Failed to run bindfs to set up test: %s", error->message);
       g_test_skip (skip_message);
+
       g_free (skip_message);
       g_clear_error (&error);
+
+      g_clear_object (&mount_monitor);
+      g_free (dir_mountpoint);
+      g_free (file_in_mount);
+      g_free (dir_to_mount);
+      g_free (curdir);
+      g_free (fusermount);
+      g_free (bindfs);
+
       return;
     }
 
@@ -170,6 +189,18 @@ test_filesystem_readonly (gconstpointer with_mount_monitor)
   if (! g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY))
     {
       g_test_skip ("Failed to create readonly file needed to proceed further with the test");
+
+      g_clear_object (&file_info);
+      g_clear_object (&mounted_file);
+      g_free (file_in_mountpoint);
+      g_clear_object (&mount_monitor);
+      g_free (dir_mountpoint);
+      g_free (file_in_mount);
+      g_free (dir_to_mount);
+      g_free (curdir);
+      g_free (fusermount);
+      g_free (bindfs);
+
       return;
     }
 
@@ -225,8 +256,7 @@ main (int argc, char *argv[])
 
   g_test_init (&argc, &argv, NULL);
 
-  g_test_bug_base ("http://bugzilla.gnome.org/");
-  g_test_bug ("787731");
+  g_test_bug ("https://bugzilla.gnome.org/show_bug.cgi?id=787731");
 
   g_test_add_data_func ("/g-file-info-filesystem-readonly/test-fs-ro",
                         GINT_TO_POINTER (FALSE), test_filesystem_readonly);
